//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DOMINODEVICEMANAGER_H
#define DOMINODEVICEMANAGER_H

#include <QObject>
#include <QSharedPointer>
#include <QVariant>
#include <QMap>
#include <QList>

#include "DeviceEntry.h"

class DominoDevice;
class QHostAddress;
class DominoSettings;
class EventData;
class DominoData;

class DeviceDatabase : public QMap<QString, DeviceEntry> {};

class DominoDeviceManager : public QObject
{
    Q_OBJECT
public:
    explicit DominoDeviceManager(QObject *parent = 0);
    virtual ~DominoDeviceManager();
//    DeviceProperties getProperties() const;
    int getChannelMaxSampleCount() const;
    bool isConnected() const;
//    QtMregDevice::DeviceId getDeviceId() const;
    QString getDeviceIdStr() const;

signals:
    void setupChanged(const DominoSettings &);
    void deviceConnected();
    void deviceDisconnected();
//    void devicePropertiesChanged(const DeviceProperties &);
    void tempChanged(QString);
    void numberOfChannelChanged(int);
    void adcCalibrationStatusChanged(bool);
    void adcCalibrationStart();
    void dataAcquired(const EventData &);
    void monitorEventChanged(const EventData &);
    void clearEventNumber();

public slots:
    void setHostAddressList(const QList<QHostAddress> &);
//    void clearEventNumber();

private slots:
    void onDeviceConnect();
    void onDeviceDisconnect();
    void onDeviceTempChanged(double);
    void onDeviceNumberOfChannelChanged(int);
    void onDeviceAdcCalibrationStatusChanged();
    void onDeviceAdcCalibrationStart();
    void onDeviceDataAcquired(const DominoData &dominoData);
    void clearFullEventNumber();

private:
    void maybeSendMonitorEvent(const EventData &eventData);

    DeviceDatabase devices;
//    QMap<QString, double> temperatures;
//    DeviceProperties * const properties;
    EventData * const eventData;
    EventData * const monitorEvent;
};

#endif // DOMINODEVICEMANAGER_H
