//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "EventWriter.h"
#include <QFile>
#include <QTextStream>
#include <QDateTime>
#include <QDebug>
#include <QTimer>
#include <QDir>
#include "EventData.h"
#include "util/QDateTimeCompat.h"

const int dataFileFlushPeriodMs = 5000;

EventWriter::EventWriter(QObject *parent) :
    QObject(parent),
    DataFile(new QFile(this)),
    DataOut(new QTextStream(DataFile)),
    flushTimer(new QTimer(this))
{
    DataFileName = QDir::homePath()+QDir::separator()+"adc64.txt";
    connect(flushTimer, SIGNAL(timeout()), SLOT(flush()));
    flushTimer->start(dataFileFlushPeriodMs);
}

EventWriter::~EventWriter()
{
    flush();
    delete DataOut;
    delete DataFile;
}

void EventWriter::setFileName(const QString &str)
{
    DataFileName = str;
    if (DataFile->isOpen())
        DataFile->close();
    DataFile->setFileName(DataFileName);
    if (DataFile->open(QFile::WriteOnly | QFile::Truncate)) {
//        DataOut.setDevice(&DataFile);
    }
}

void EventWriter::close()
{
    DataFile->close();
}

void EventWriter::flush()
{
    if (DataFile->isOpen())
        DataFile->flush();
}

void EventWriter::SaveData(const EventData &eventData)
{
    const QDateTime &current = QDateTime::currentDateTime();

    if (QDateTimeMsescTo(eventData.timestamp, current) > 1000) { // data too old
        qDebug() << QString("EventWriter: event %1 dropped (old)").arg(eventData.getEventNumberStr());
        return;
    }
    //    qDebug() << QString("EventWriter: %1 channels").arg(dominoData.Data.size());
    int channelOffset = 0;
    *DataOut << "Event " << eventData.FullEventNumber << " " << QString("%1").arg(comp_time.currentDateTime().toTime_t()) << "\n";
    Q_FOREACH(const DominoData &dominoData, eventData) {
        Q_FOREACH(int i, dominoData.rawData.keys()) {
            if(dominoData.rawData[i].size()>0)     //don't save empty channels
            {
                *DataOut << channelOffset+i+1 << " ";
                for (size_t j=0; j<dominoData.rawData[i].size(); j++)
                {
                    //            DataOut << QString("%1").arg(dominoData.Data[i][j], 0, 'e') << " ";  //save data as double
                    *DataOut << QString("%1").arg(dominoData.rawData[i][j], 0) << " ";  //save data as int
                }
                *DataOut << "\n";
            }
        }
        channelOffset += dominoData.rawData.size();
    }
    qDebug() << QString("EventWriter: event %1").arg(eventData.getEventNumberStr());
}
