#ifndef DOMINODEVICE_H
#define DOMINODEVICE_H

#include <QObject>
#include <QTimer>
#include <QDateTime>

#include "dominodata.h"
#include "cellcalibration.h"
#include "dominosettings.h"
#include "regio/qtmregdevice.h"

class DominoDevice : public QtMregDevice
{
    Q_OBJECT
public:
    typedef enum {TRIG_NEGATIVE = 0, TRIG_POSITIVE = 1} TrigPolarity;
    DominoDevice(QObject *parent = 0);
    virtual ~DominoDevice();
    bool isRunning();
    void SaveData(const DominoData &dominoData);
    DominoData getDominoData() const {return dominoData;}
//    DominoRealData getDominoRealData() const {return dominoData.Data;}
    int getNumberOfChannels() const;
    int getChannelMaxSampleCount() const;
    void SetDelayAfterTrig(int del);
    void DacWrite(int dacnum, int ch, int data);
    void DacMax550Write(quint8 addr, quint8 data);
    void ChannelSetup(int ch, bool thr_trig_en, TrigPolarity thr_trig_polarity, TrigPolarity zs_thr_polarity);
    void ChnSetThreshold(int ch, int thr);
    void ChnSetZsThreshold(int ch, int thr);
    void WritePCAReg(const reg_pca_struct &reg_pca);
    void WriteAD5622Command(const ad5622operation &ad5622op);
    void SetAmpBoard(const quint16 board);
    bool getAdcCalibrationStatus() { return adcCalibrationStatus; }

Q_SIGNALS:
    void numberOfChannelChanged(int);
    void adcCalibrationStatusChanged();
    void dataAcquired(DominoData);
    void startAdcCalibration();

public Q_SLOTS:
    void on_setupChanged(const DominoSettings &);
    void clearEventNumber() { dominoData.EventNumber=0; }

private:
    QTimer * const pollTimer; // create without parent
    DominoData dominoData;
    DominoSettings setup;
    DominoSettings settingsCache;
    bool settingsCacheValid;
    bool saveRunState;
    bool adcCalibrationStatus;
    QDateTime lastStatusTimestamp;

    void RestartRun();
    void StartRun();
    void StopRun();
    void ReadAllChMem();
    void writeSettings(const DominoSettings &settings);
    void WriteTrigSetup(reg_trig_ctrl trigsetup);
    void ReadChMem(int ch, const quint32 addr_offset);
    quint16 ReadChReg(int ch, quint16 addr);
    void WriteChReg(int ch, quint16 addr, quint16 data);
    bool AdcSpiWrite(quint8 reg, quint8 data, bool readback);
    QVector<quint16> AdcSpiRead(quint8 reg);
    void SetAdcCustomPattern(quint16 data);
    void SetAdcCustomPatternMode(bool mode);
    void SetTrigDelay(int delay);
    void DetectNumberOfAdc64boards();
    int RegWRTest(int NCycle); 
    bool testChannelRegisters();
    QVector<bool> AdcPatternTest();
    QVector<bool> TestAdcCustomPattern(const quint16);
    void ChangeAdcClkDelay(bool, quint8, quint16);
    void ResetAdcClkDelay();
    void ResetAdc();
    bool FindAdcClkDelay();
    void SetAdcClkDelay(const QVector<int>);
    int FindTruePlateauCenter(const QVector<bool>);
    bool isMaster();

    // EVADC, ADCM16-3 boards have AD9252
    void ad9252_spi_write(quint16 addr, quint8 data, int device);
    quint16 ad9252_spi_read(quint16 addr, int device);
    void setup_adc(int device);

private Q_SLOTS:
    void on_dominoDeviceConnected();
    void on_timerPoll();
    void on_startAdcCalibration();
};

#endif // DOMINODEVICE_H
