//
//    Copyright 2010 Alexey Shutov, Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DOMINOSETTINGS_H
#define DOMINOSETTINGS_H

#include <QMap>
#include <QVariant>
#include <QMetaType>

typedef union {
    quint16 all;
    struct {
        unsigned int timer_en: 1;
        unsigned int threshold_en: 1;
        unsigned int nim_en: 1;
        unsigned int unused2: 13;
    } bit;
} reg_trig_ctrl;

typedef union {
    quint16 all;
    struct {
        unsigned int polarity1: 1;
        unsigned int shaper1_1: 1;
        unsigned int shaper1_2: 1;
        unsigned int shaper1_3: 1;
        unsigned int gain1_1: 1;
        unsigned int gain1_2: 1;
        unsigned int preamp_en1: 1;
        unsigned int unused1: 1;
        unsigned int polarity2: 1;
        unsigned int shaper2_1: 1;
        unsigned int shaper2_2: 1;
        unsigned int shaper2_3: 1;
        unsigned int gain2_1: 1;
        unsigned int gain2_2: 1;
        unsigned int preamp_en2: 1;
        unsigned int unused2: 1;
    } bit;
} reg_pca_struct;


struct ad5622operation{
    quint16 data_command;
    quint16 data_baseline;
    quint16 data_level;
    ad5622operation() : data_command(0), data_baseline(0), data_level(0) {}
/*
    unsigned int power_down_modes: 2;
    unsigned int reserved1: 2;
    unsigned int read_operation: 1;
    unsigned int hw_address: 2;
    unsigned int reserved2: 5;
    unsigned int unused: 8;
*/
};

enum {
    AMP_BOARD_NONE    = 0x0,
    AMP_BOARD_PCA     = 0x1,
    AMP_BOARD_AD5622  = 0x2
};


class DominoSettings
{
public:
//    Q_ENUMS(Mode)
    typedef enum {NormalMode = 0, AmpCalMode = 1} Mode;
    DominoSettings();
    void resize(int n);

    QMap<int, bool> en;
    QMap<int, bool> trig;
    QMap<int, int> thr;
    QMap<quint64, QMap<int, int> > baseline;  //device serial id - channel number - baseline

    int nch;
    bool run;
    Mode mode;
    int readcellnumber;
    int trigdelay;
    bool zs_en;
    int zs_thr;
    bool invert_thr_trig;
    bool invert_zs_thr;
    reg_trig_ctrl trigsetup;
    reg_pca_struct reg_pca;
    ad5622operation ad5622op;
    quint16 amp_board_set;
    // EVADC only
    //    int writecellnumber;
    //    int analogDelay;
};
Q_DECLARE_METATYPE(DominoSettings)
Q_DECLARE_METATYPE(DominoSettings::Mode)

//template <class T>
//QVariantMap qMapToVariant(QMap<int, T> map)
//{
//    QVariantMap result;
//    Q_FOREACH(int i, map.keys())
//        result[QString::number(i)] = map.value(i);
//    return result;
//}

#endif // DOMINOSETTINGS_H
