#ifndef MAINWINDOW_H
#define MAINWINDOW_H


#include <QMainWindow>
#include "dominosettings.h"
#include "dominodata.h"
#include <QMap>
#include <QDateTime>

namespace Ui {
    class MainWindow;
}

class QLabel;
class DominoDeviceManager;
class DominoData;
class EventData;
class EventWriter;
class QThread;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

Q_SIGNALS:
    void setupChanged(const DominoSettings &);

protected:
    void changeEvent(QEvent *e);

private:
    void setupDevice();
    void readSettings();
    void writeSettings();
    virtual void closeEvent(QCloseEvent *event);
    void setBoardHostName();
    void restoreFormValues();

    Ui::MainWindow * const ui;
    QLabel * const statusLabel;
    QLabel * const evLabel;
    QLabel * const tempLabel;
    QLabel * const idLabel;
    DominoDeviceManager * const dominoDevice;
    DominoSettings setup;
    EventData * const lastEventData;
    QDateTime scopeDisplayTime;
    EventWriter * const eventWriter;
    QThread * const deviceThread;
    QThread * const writerThread;

private Q_SLOTS:
    void on_checkBoxTrigThreshold_toggled(bool checked);
    void on_checkBoxTrigNIM_toggled(bool checked);
    void on_checkBoxTrigTime_toggled(bool checked);
    void on_comboBoxOctalADC_currentIndexChanged(int index);
    void onDeviceConnect();
    void onDeviceDisconnect();
    void setStatus(const QString &str);
    void setTemp(const QString &str);
    void on_pushButtonConnect_clicked();
    void on_spinBoxAfterTrig_valueChanged(int );
    void on_checkBoxTr8_toggled(bool checked);
    void on_checkBoxTr7_toggled(bool checked);
    void on_checkBoxTr6_toggled(bool checked);
    void on_checkBoxTr5_toggled(bool checked);
    void on_checkBoxTr4_toggled(bool checked);
    void on_checkBoxTr3_toggled(bool checked);
    void on_checkBoxTr2_toggled(bool checked);
    void on_checkBoxTr1_toggled(bool checked);
    void on_checkBoxE8_toggled(bool checked);
    void on_checkBoxE7_toggled(bool checked);
    void on_checkBoxE6_toggled(bool checked);
    void on_checkBoxE5_toggled(bool checked);
    void on_checkBoxE4_toggled(bool checked);
    void on_checkBoxE3_toggled(bool checked);
    void on_radioButtonNormMode_toggled(bool checked);
    void on_spinBoxT8_valueChanged(int );
    void on_spinBoxT7_valueChanged(int );
    void on_spinBoxT6_valueChanged(int );
    void on_spinBoxT5_valueChanged(int );
    void on_spinBoxT4_valueChanged(int );
    void on_spinBoxT3_valueChanged(int );
    void on_spinBoxT2_valueChanged(int );
    void on_checkBoxW2D_toggled(bool checked);
    void on_checkBoxE2_toggled(bool checked);
    void on_checkBoxE1_toggled(bool checked);
    void on_spinBoxReadCNT_valueChanged(int );
    void on_spinBoxT1_valueChanged(int );
    void on_pushButtonStop_clicked();
    void on_pushButtonStart_clicked();
//    void DoTimer();
//    void DoStopTimer();
    void on_actionQuit_activated();
    void on_actionDocs_activated();
    void on_actionCheckForUpdates_activated();
    void on_actionAbout_activated();
    void on_actionAboutQt_activated();
    void setNumberOfChannels(int n);
    void setAdcCalibrationStatus(bool status);
    void setAdcCalibrationStart();
    void setChannelEnabled(int n, bool value);
    void setChannelTrig(int n, bool value);
    void setChannelThreshold(int n, int value);
    void DrawScope(const EventData &);
//    void SaveData(const DominoData &dominoData);
    void enableRunControlButtons();
    void on_spinBoxZSThr_valueChanged(int arg1);
    void on_pushButtonLoadBaseline_clicked();
    void on_spinBoxAmpBL_valueChanged(int arg1);
    void on_spinBoxAmpLVL_valueChanged(int arg1);
    void on_groupBoxShaperSetup_toggled(bool newval);
    void on_groupBoxAmp64Setup_toggled(bool newval);
    void on_comboBoxThrTrig_currentIndexChanged(const QString &arg1);
    void on_groupBoxZS_toggled(bool checked);
    void on_comboBoxZSThr_currentIndexChanged(const QString &arg1);
};

#endif // MAINWINDOW_H
